
/************************/
/* Including the C libs */

#include <stdio.h>
#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "xgdatamacros.h"
#include "xgminint.h"

void PrintUsage(char *string)
{
  printf("Usage: %s [options]\n",string);
  printf("\t-d dumpfile[.dmp]:\n\t\tSpecify dump file name. Default is output.dmp\n");
  printf("\t-display displayname:\n\t\tSpecify the display. Default is DISPLAY environment variable.\n");
  printf("\t-dp n:\n\t\tSpecify the dump period. Default is 0 (no periodic dumps)\n");
  printf("\t-exit:\n\t\tExit after done with iterations\n");
  printf("\t-h:\n\t\tThis help message\n");
  printf("\t-i inputfile[.inp]:\n\t\tSpecify input file name. Default is no input file\n");
  printf("\t-nox:\n\t\tRun without X\n");
  printf("\t-p epsfile[.eps]:\n\t\tSpecify ps file name. Default is output.eps\n");
  printf("\t-s n:\n\t\tSpecify the number of iterations. Default is 0 (run forever)\n");
  printf("\t-u n:\n\t\tSpecify the number of iterations per X update. Default is 1\n");
}

void PrintUnknownOption(char *string, char *string2)
{
  printf("Unknown option specified: %s\n",string);
  PrintUsage(string2);
}

void PrintNoArg(char *string, char *string2)
{
  printf("No argument given to option: %s\n",string);
  PrintUsage(string2);
}



void XGInit(int argc, char **argv, double *t)
{
  int index, endianInt = 1;
  char *endianChar;
  
  /*******************/
  /* The C stuff */
  
  theTimeStep = t;
  theRunFlag = FALSE;
  theStepFlag = FALSE;
  theExitFlag = FALSE;
  theNumberOfSteps = 0;
  iterationsPerXUpdate = 1;
  theCurrentStep = 1;
  theDumpPeriod = 0;
  WasDumpFileGiven = FALSE;
  WasInputFileGiven = FALSE;
  theExitFlag = FALSE;

  /***********************************/
  /* Checking the endianness of host */

  endianChar = (char*)&endianInt;

  if (*endianChar == (char)1) {
     theEndian = LITTLE;
  } else {
     theEndian = BIG;
  }

  /*****************************************/
  /* Getting the code and input file names */

  sprintf(theCodeName, "%s", argv[0]);
  theInputFile[0]='\0';

/* Try to read the input parameters, i.e. -i -d -dp, etc. options */
   for(index=1;index<argc;index++) {
    if(argv[index][0] != '-') {
      PrintUnknownOption(argv[index],argv[0]);
      exit(1);
    }
    switch(argv[index][1]) {
    case 'd':
      if(argv[index][2]!='\0' && argv[index][2]!='p' && 
	 strcmp(argv[index],"-display")) {
	PrintUnknownOption(argv[index],argv[0]);
	exit(1);
      }
      if(index+1 >= argc) {
	PrintNoArg(argv[index],argv[0]);
	exit(1);
      }
      switch(argv[index][2]) {
      case '\0':
	strcpy(theDumpFile, argv[++index]);
	if(access(theDumpFile,F_OK)) {
	  strcat(theDumpFile, theDumpExtension);
	  if(access(theDumpFile,F_OK)) {
	    printf("Error: %s does not exist\n",theDumpFile);
	    exit(1);
	  }
	}

	printf("Dump file is '%s'\n",theDumpFile);
	WasDumpFileGiven = TRUE;
	break;
      case 'p':
	theDumpPeriod = atoi(argv[++index]);
	printf("Dumping every %d steps\n",theDumpPeriod);
	break;
      case 'i':
	printf("MiniXGRAFIX doesn't display.");
	break;
      }
      break;
    case 'e':
      if (!strcmp(argv[index],"-exit")) {
	theExitFlag = TRUE;
      } else {
	PrintUnknownOption(argv[index],argv[0]);
	exit(1);
      }
      break;
    case 'h':
      PrintUsage(argv[0]);
      exit(0);
      break;
    case 'i':
      if(argv[index][2]!='\0') {
	PrintUnknownOption(argv[index],argv[0]);
	exit(1);
      }
      if(index+1 >= argc) {
	PrintNoArg(argv[index],argv[0]);
	exit(1);
      }
      strcpy(theInputFile, argv[++index]);

      if(access(theInputFile,F_OK)) {
	strcat(theInputFile,theInputExtension);
	if(access(theInputFile,F_OK)) {
	  printf("Error: %s does not exist\n",theInputFile);
	  exit(1);
	}
      }

      WasInputFileGiven = TRUE;
      printf("\nInput file is '%s'\n",theInputFile);
      break;
    case 'n':
      if(!strcmp(argv[index],"-nox")) {
	printf("Running without X\n");
      } else {
	PrintUnknownOption(argv[index],argv[0]);
	exit(1);
      }
      break;
    case 'p':
      if(argv[index][2]!='\0') {
	PrintUnknownOption(argv[index],argv[0]);
	exit(1);
      }
      if(index+1 >= argc) {
	PrintNoArg(argv[index],argv[0]);
	exit(1);
      }
      strcpy(theEPSFile, argv[++index]);
      printf("Postscript file is '%s'\n",theEPSFile);
      break;
    case 's':
      if(argv[index][2]!='\0') {
	PrintUnknownOption(argv[index],argv[0]);
	exit(1);
      }
      if(index+1 >= argc) {
	PrintNoArg(argv[index],argv[0]);
	exit(1);
      }
      theNumberOfSteps = atoi(argv[++index]);
      printf("Running for %d steps\n",theNumberOfSteps);
      break;

	 case 'u':  /*  set the number of iterations per X update */
      if(argv[index][2]!='\0') {
		  PrintUnknownOption(argv[index],argv[0]);
		  exit(1);
      }
      if(index+1 >= argc) {
		  PrintNoArg(argv[index],argv[0]);
		  exit(1);
      }
      iterationsPerXUpdate = atoi(argv[++index]);
		if(iterationsPerXUpdate < 1) iterationsPerXUpdate = 1;
      printf("Iterations per X update: %d steps\n",iterationsPerXUpdate);
      break;

    default:
      PrintUnknownOption(argv[index],argv[0]);
      exit(1);
      break;
    }      
  }      
/* End of trying to read parameters */

  if (!WasInputFileGiven) {
    strcpy(theEPSFile,theDefaultEPSFile);
    strcpy(theDumpFile,theDefaultDumpFile);
  } else if (!WasDumpFileGiven) {
    strncpy(theEPSFile,theInputFile,findlen(theInputFile));
    strncpy(theDumpFile,theInputFile,findlen(theInputFile));
    strcat(theEPSFile,theEPSExtension);
    strcat(theDumpFile,theDumpExtension);
  } else {
    strncpy(theEPSFile,theInputFile,findlen(theInputFile));
    strcat(theEPSFile,theEPSExtension);
  }

}

/* A procedure called by XGInit to determine the rootlength of filenames */
int findlen(char *string){
 int i, len;

 len = strlen(string);
 i = len-1;
 while (string[i] != '.' && i  >=0) {
   if(string[i]=='/') return len;
   i = i - 1;
 }
 if(i>0)  return (i);
 return len;

}


void XGStart()
{
  while (!theExitFlag) {
      
	 while ((theNumberOfSteps==0 || theCurrentStep<=theNumberOfSteps) ) {
		XGMainLoop();
		if (theDumpPeriod!=0 && theCurrentStep%theDumpPeriod==0) {
		  Dump(theDumpFile);
		}
		theCurrentStep++;
	 }
	 if (theDumpPeriod!=0 && theCurrentStep%theDumpPeriod==0)
		Dump(theDumpFile);
	 if(theCurrentStep>theNumberOfSteps) {
		Quit();
		exit(0);
	 }	
  }
  Quit();
  exit(0);
}

